# -*- coding: utf-8 -*-

from freezegun import freeze_time
from imio.smartweb.core import config
from imio.smartweb.core.contents.rest.base import BaseEndpoint
from imio.smartweb.core.contents.rest.directory.endpoint import DirectoryEndpoint
from imio.smartweb.core.contents.rest.events.endpoint import EventsEndpoint
from imio.smartweb.core.contents.rest.news.endpoint import NewsEndpoint
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_ACCEPTANCE_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.tests.utils import get_json
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession
from unittest.mock import patch
from urllib.parse import urlparse
from urllib.parse import parse_qs
from zope.component import queryMultiAdapter
from zope.publisher.browser import TestRequest

import json
import requests_mock
import transaction

# Get full string when string are too long in assertEqual errors
# __import__('sys').modules['unittest.util']._MAX_LENGTH = 999999999


class SectionsFunctionalTest(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_ACCEPTANCE_TESTING

    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.json_rest_directory = get_json("resources/json_rest_directory.json")
        self.json_rest_events = get_json("resources/json_rest_events.json")
        self.json_used_iam_voc = get_json(
            "resources/json_used_iam_vocabularies_jobseeker_tourist.json"
        )
        self.json_rest_news = get_json("resources/json_rest_news.json")
        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})

        self.rest_directory = api.content.create(
            container=self.portal,
            type="imio.smartweb.DirectoryView",
            title="directory view",
        )
        api.content.transition(self.rest_directory, "publish")

        self.rest_events = api.content.create(
            container=self.portal,
            type="imio.smartweb.EventsView",
            title="events view",
        )
        self.rest_events.selected_agenda = "64f4cbee9a394a018a951f6d94452914"
        self.rest_events.selected_event_types = ["event-driven"]
        api.content.transition(self.rest_events, "publish")

        self.rest_news = api.content.create(
            container=self.portal,
            type="imio.smartweb.NewsView",
            title="news view",
        )
        self.rest_news.selected_news_folder = "64f4cbee9a394a018a951f6d94452914"
        api.content.transition(self.rest_news, "publish")

        transaction.commit()

    def tearDown(self):
        self.api_session.close()

    @freeze_time("2021-09-14 8:00:00")
    def test_convert_cached_image_scales(self):
        endpoint = BaseEndpoint(self.portal, self.request)
        modified_hash = "78fd1bab198354b6877aed44e2ea0b4d"

        item = {"@id": "http://host.com/content", "image": {"scales": {}}}
        endpoint.convert_cached_image_scales(item, modified_hash)
        self.assertNotIn("image", item.keys())
        self.assertEqual(
            item["image_preview_scale"],
            "http://host.com/content/@@images/image/preview?cache_key=78fd1bab198354b6877aed44e2ea0b4d",
        )
        self.assertEqual(
            item["image_extralarge_scale"],
            "http://host.com/content/@@images/image/extralarge?cache_key=78fd1bab198354b6877aed44e2ea0b4d",
        )
        self.assertEqual(
            item["image_affiche_scale"],
            "http://host.com/content/@@images/image/affiche?cache_key=78fd1bab198354b6877aed44e2ea0b4d",
        )

        item = {"@id": "http://host.com/content", "logo": {"scales": {}}}
        endpoint.convert_cached_image_scales(item, modified_hash, "logo", ["thumb"])
        self.assertNotIn("logo", item.keys())
        self.assertEqual(
            item["logo_thumb_scale"],
            "http://host.com/content/@@images/logo/thumb?cache_key=78fd1bab198354b6877aed44e2ea0b4d",
        )

        item = {
            "@id": "http://host.com/content",
            "image": {"scales": {}},
            "logo": {"scales": {}},
        }
        endpoint.convert_cached_image_scales(item, modified_hash, "logo", ["thumb"])
        self.assertEqual(len(item.keys()), 3)
        endpoint.convert_cached_image_scales(item, modified_hash)
        self.assertEqual(len(item.keys()), 5)

    def test_get_extra_params(self):
        request = TestRequest(
            form={"taxonomy_contact_category": '("token")', "topics": "education"}
        )
        endpoint = DirectoryEndpoint(self.portal, request)
        params = endpoint.get_extra_params([])
        self.assertEqual(
            params,
            [
                'taxonomy_contact_category=("token")',
                "topics=education",
                "translated_in_en=1",
            ],
        )

        request = TestRequest(
            form={"topics": "education", "metadata_fields": ["topics", "category"]}
        )
        endpoint = DirectoryEndpoint(self.portal, request)
        params = endpoint.get_extra_params([])
        self.assertEqual(
            params,
            [
                "topics=education",
                "metadata_fields=topics",
                "metadata_fields=category",
                "translated_in_en=1",
            ],
        )

    @requests_mock.Mocker()
    def test_call_directory(self, m):
        # We don't want to make real request on authentic sources if some Mocks are missing
        rest_directory = api.content.create(
            container=self.portal,
            type="imio.smartweb.DirectoryView",
            title="My directory view",
        )
        # react additionnal fields request.
        request = TestRequest(
            form={
                "taxonomy_contact_category_for_filtering": ("token"),
                "topics": "education",
            }
        )
        endpoint = DirectoryEndpoint(rest_directory, request)
        url = endpoint.query_url
        self.assertEqual(
            url,
            "http://localhost:8080/Plone/@search?"
            "selected_entities=396907b3b1b04a97896b12cc792c77f8&"
            "portal_type=imio.directory.Contact&"
            "metadata_fields=facilities&"
            "metadata_fields=taxonomy_contact_category&"
            "metadata_fields=topics&"
            "metadata_fields=has_leadimage&"
            "fullobjects=1&"
            "sort_on=sortable_title&"
            "b_size=20&"
            "taxonomy_contact_category_for_filtering=token&"
            "topics=education&"
            "translated_in_en=1",
        )
        m.get(url, text=json.dumps({}))
        call = endpoint()
        self.assertEqual(call, {})

        rest_directory.nb_results = 30
        url = endpoint.query_url
        self.assertNotIn("b_size=20", url)
        self.assertIn("b_size=30", url)

        json_contact_category_raw_mock = get_json(
            "resources/json_contact_category_raw_mock.json"
        )
        with patch(
            "imio.smartweb.core.vocabularies.get_wca_token", return_value="kamoulox"
        ):
            url = f"{config.DIRECTORY_URL}/@vocabularies/collective.taxonomy.contact_category"
            m.get(url, text=json.dumps(json_contact_category_raw_mock))

            rest_directory.selected_categories = ["hlsm9bijb1", "9kgcmrj4lu"]
            url = endpoint.query_url
            self.assertEqual(
                url,
                "http://localhost:8080/Plone/@search?"
                "selected_entities=396907b3b1b04a97896b12cc792c77f8&"
                "portal_type=imio.directory.Contact&"
                "metadata_fields=facilities&"
                "metadata_fields=taxonomy_contact_category&"
                "metadata_fields=topics&"
                "metadata_fields=has_leadimage&"
                "fullobjects=1&"
                "sort_on=sortable_title&"
                "b_size=30&"
                "taxonomy_contact_category.query=hlsm9bijb1&"
                "taxonomy_contact_category.query=9kgcmrj4lu&"
                "taxonomy_contact_category.operator=or&"
                "taxonomy_contact_category_for_filtering=token&"
                "topics=education&"
                "translated_in_en=1",
            )

    def test_render_rest_directory(self):
        view = queryMultiAdapter((self.rest_directory, self.request), name="view")
        self.assertIn("<smartweb-annuaire", view())

    def test_rest_directory_results(self):
        params = {}
        with patch(
            "imio.smartweb.core.contents.rest.base.get_json",
            return_value=self.json_rest_directory,
        ) as mypatch:
            response = self.api_session.get("/directory-view/@results", params=params)
            patch_url = mypatch.return_value.get("@id")
            patch_urlparsed = urlparse(patch_url)
            patch_directory = parse_qs(patch_urlparsed.query)
            response_url = response.json().get("@id")
            response_urlparsed = urlparse(response_url)
            response_directory = parse_qs(response_urlparsed.query)
            self.assertEqual(patch_directory, response_directory)

    @requests_mock.Mocker()
    @freeze_time("2021-11-15")
    def test_call_events(self, m):
        self.rest_events.selected_agenda = "64f4cbee9a394a018a951f6d94452914"
        self.rest_events.selected_event_types = ["event-driven"]
        endpoint = EventsEndpoint(self.rest_events, self.request)
        url = endpoint.query_url
        self.maxDiff = None
        self.assertEqual(
            url,
            "http://localhost:8080/Plone/@events?"
            "selected_agendas={}&"
            "metadata_fields=category&"
            "metadata_fields=topics&"
            "metadata_fields=start&"
            "metadata_fields=end&"
            "metadata_fields=has_leadimage&"
            "metadata_fields=UID&"
            "event_dates.query=2021-11-15&"
            "event_dates.range=min&"
            "sort_on=event_dates&"
            "fullobjects=1&"
            "b_size=20&"
            "event_type=event-driven&"
            "translated_in_en=1".format(self.rest_events.selected_agenda),
        )
        m.get(url, text=json.dumps({}))
        call = endpoint()
        self.assertEqual(call, {})

        self.rest_events.nb_results = 30
        url = endpoint.query_url
        self.assertNotIn("b_size=20", url)
        self.assertIn("b_size=30", url)

    def test_render_rest_events(self):
        view = queryMultiAdapter((self.rest_events, self.request), name="view")
        self.assertIn("<smartweb-events", view())

    @freeze_time("2021-11-15")
    def test_rest_events_results(self):
        params = {}
        # mock get_json from where we import/use it
        with patch(
            "imio.smartweb.core.contents.rest.base.get_json",
            return_value=self.json_rest_events,
        ) as mypatch:
            patch_url = mypatch.return_value.get("@id")
            patch_urlparsed = urlparse(patch_url)
            patch_selected_agendas = parse_qs(patch_urlparsed.query)[
                "selected_agendas"
            ][0]
            response = self.api_session.get("/events-view/@results", params=params)
            response_url = response.json().get("@id")
            response_urlparsed = urlparse(response_url)
            response_selected_agendas = parse_qs(response_urlparsed.query)[
                "selected_agendas"
            ][0]
            self.assertEqual(patch_selected_agendas, response_selected_agendas)

    @requests_mock.Mocker()
    def test_call_news(self, m):
        endpoint = NewsEndpoint(self.rest_news, self.request)
        url = endpoint.query_url
        self.maxDiff = None
        self.assertEqual(
            url,
            "http://localhost:8080/Plone/@search?"
            "selected_news_folders={}&"
            "portal_type=imio.news.NewsItem&"
            "metadata_fields=category&"
            "metadata_fields=topics&"
            "metadata_fields=has_leadimage&"
            "metadata_fields=UID&"
            "sort_on=effective&"
            "sort_order=descending&"
            "b_size=20&"
            "fullobjects=1&"
            "translated_in_en=1".format(self.rest_news.selected_news_folder),
        )
        m.get(url, text=json.dumps({}))
        call = endpoint()
        self.assertEqual(call, {})

        self.rest_news.nb_results = 30
        url = endpoint.query_url
        self.assertNotIn("b_size=20", url)
        self.assertIn("b_size=30", url)

    def test_render_rest_news(self):
        view = queryMultiAdapter((self.rest_news, self.request), name="view")
        self.assertIn("<smartweb-news", view())

    def test_rest_news_results(self):
        params = {}
        with patch(
            "imio.smartweb.core.contents.rest.base.get_json",
            return_value=self.json_rest_news,
        ) as mypatch:
            response = self.api_session.get("/news-view/@results", params=params)
            patch_url = mypatch.return_value.get("@id")
            patch_urlparsed = urlparse(patch_url)
            patch_selected_news_folders = parse_qs(patch_urlparsed.query)[
                "selected_news_folders"
            ][0]
            response_url = response.json().get("@id")
            response_urlparsed = urlparse(response_url)
            response_selected_news_folders = parse_qs(response_urlparsed.query)[
                "selected_news_folders"
            ][0]
            self.assertEqual(
                patch_selected_news_folders, response_selected_news_folders
            )



# -*- coding: utf-8 -*-

from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from plone import api
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from plone.app.textfield.value import RichTextValue
from zope.component import queryMultiAdapter

import json


class TestSearch(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            id="folder",
        )
        api.content.transition(self.folder, "publish")

    def test_search(self):
        self.request.form["SearchableText"] = "kamoulox"
        search = queryMultiAdapter((self.portal, self.request), name="search")
        self.assertEqual(len(search.results()), 0)

        page = api.content.create(
            container=self.folder,
            type="imio.smartweb.Page",
            title="kamoulox",
        )

        self.assertEqual(len(search.results()), 1)

        section_text = api.content.create(
            container=page,
            type="imio.smartweb.SectionText",
            title="kamoulox",
        )
        section_text.text = RichTextValue("<p>kamoulox</p>", "text/html", "text/html")
        section_text.reindexObject()

        # Some sections (like SectionText) are not searchable
        self.assertEqual(len(search.results()), 1)

        api.content.create(
            container=self.portal,
            type="imio.smartweb.PortalPage",
            title="kamoulox",
        )
        self.assertEqual(len(search.results()), 2)

    def test_search_result_option(self):
        search = queryMultiAdapter((self.portal, self.request), name="search")
        options = json.loads(search.get_search_result_option())
        self.assertDictEqual(options, {"directory": True, "events": True, "news": True})



# -*- coding: utf-8 -*-

from imio.smartweb.core.interfaces import IImioSmartwebCoreLayer
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.tests.utils import make_named_image
from plone import api
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from plone.namedfile.file import NamedBlobImage
from plone.testing.zope import Browser
from zope.interface import alsoProvides

import transaction


class TestSocial(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        alsoProvides(self.request, IImioSmartwebCoreLayer)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_images_urls(self):
        folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="Folder",
            id="folder",
        )
        api.content.transition(folder, "publish")
        page = api.content.create(
            container=folder, type="imio.smartweb.Page", title="Page", id="page"
        )
        page.image = NamedBlobImage(**make_named_image())
        api.content.transition(page, "publish")
        transaction.commit()

        browser = Browser(self.layer["app"])
        browser.open(self.portal.absolute_url())
        content = browser.contents
        self.assertIn(
            '<meta content="http://nohost/plone/++resource++plone-logo.svg" property="og:image"/>',
            content,
        )
        self.assertIn(
            '<span itemprop="image">http://nohost/plone/++resource++plone-logo.svg</span>',
            content,
        )

        browser.open(folder.absolute_url())
        content = browser.contents
        self.assertIn(
            '<meta content="http://nohost/plone/++resource++plone-logo.svg" property="og:image"/>',
            content,
        )
        self.assertIn(
            '<span itemprop="image">http://nohost/plone/++resource++plone-logo.svg</span>',
            content,
        )

        scales = page.restrictedTraverse("@@images")
        image = scales.scale("image", scale="vignette")
        browser.open(page.absolute_url())
        content = browser.contents
        self.assertIn(f'<meta content="{image.url}" property="og:image"/>', content)
        self.assertIn(f'<span itemprop="image">{image.url}</span>', content)

        folder.image = NamedBlobImage(**make_named_image())
        transaction.commit()
        scales = folder.restrictedTraverse("@@images")
        image = scales.scale("image", scale="vignette")
        browser.open(folder.absolute_url())
        content = browser.contents
        self.assertIn(f'<meta content="{image.url}" property="og:image"/>', content)
        self.assertIn(f'<span itemprop="image">{image.url}</span>', content)



# -*- coding: utf-8 -*-

from imio.smartweb.common.config import DESCRIPTION_MAX_LENGTH
from imio.smartweb.core.contents.sections.base import ISection
from imio.smartweb.core.contents.sections.base import Section
from imio.smartweb.locales import SmartwebMessageFactory as _
from zope.interface import implementer
from zope import schema


class ISectionLinks(ISection):
    """Marker interface and Dexterity Python Schema for SectionLinks"""

    description = schema.Text(
        title=_("Description"),
        description=_(
            "Use **text** to set text in bold. Limited to ${max} characters.",
            mapping={"max": DESCRIPTION_MAX_LENGTH},
        ),
        max_length=DESCRIPTION_MAX_LENGTH,
        required=False,
    )

    nb_results_by_batch = schema.Choice(
        title=_("Number of items per batch"),
        required=True,
        default=3,
        values=range(1, 9),
    )


@implementer(ISectionLinks)
class SectionLinks(Section):
    """SectionLinks class"""

    manage_content = True
    manage_display = True



# -*- coding: utf-8 -*-

from imio.smartweb.common.utils import get_vocabulary
from imio.smartweb.core.contents import ICirkwiView
from imio.smartweb.core.contents import IPage
from imio.smartweb.core.contents import IProcedure
from Products.Five.browser import BrowserView


class UtilsView(BrowserView):
    """ """

    def is_instancebehaviors_assignable_content(self):
        instance_behaviors_voc = get_vocabulary(
            "imio.smartweb.vocabulary.AvailableInstanceBehaviors"
        )
        if len(instance_behaviors_voc) == 0:
            return False
        return (
            IPage.providedBy(self.context)
            or IProcedure.providedBy(self.context)
            or ICirkwiView.providedBy(self.context)
        )



# -*- coding: utf-8 -*-

from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_INTEGRATION_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.viewlets.chatbot import ChatbotViewlet
from unittest import mock

import os


class TestChatbot(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]

    @mock.patch.dict(os.environ, {"CHATBOT_URL": "", "CHATBOT_ID": ""})
    def test_nochatbot(self):
        viewlet = ChatbotViewlet(self.portal, self.request, None, None)
        viewlet.update()
        self.assertFalse(viewlet.available)

    @mock.patch.dict(
        os.environ, {"CHATBOT_URL": "https://testurl", "CHATBOT_ID": "test-id"}
    )
    def test_chatbot(self):
        viewlet = ChatbotViewlet(self.portal, self.request, None, None)
        viewlet.update()
        self.assertTrue(viewlet.available)
        self.assertEqual(viewlet.chatbot_url, "https://testurl")
        self.assertEqual(viewlet.chatbot_id, "test-id")



# -*- coding: utf-8 -*-

from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_INTEGRATION_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.tests.utils import make_named_image
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.namedfile.file import NamedBlobImage
from plone.resource.interfaces import IResourceDirectory
from zope.component import getUtility
from zope.component import queryMultiAdapter


class TestIcons(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_INTEGRATION_TESTING

    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.page = api.content.create(
            container=self.portal, type="imio.smartweb.Page", id="page"
        )
        self.section = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionLinks",
            id="links",
        )

    def test_link_icon(self):
        link = api.content.create(
            container=self.section,
            type="imio.smartweb.BlockLink",
            id="link",
        )
        link.remoteUrl = "https://www.imio.be"
        view = queryMultiAdapter((self.page, self.request), name="full_view")()
        self.assertNotIn("background-image", view)

        link.image = NamedBlobImage(**make_named_image())
        view = queryMultiAdapter((self.page, self.request), name="full_view")()
        self.assertIn(
            "background-image:url(http://nohost/plone/page/links/link/@@images/image-390",
            view,
        )

        link.svg_icon = "view.directory"
        view = queryMultiAdapter((self.page, self.request), name="full_view")()
        self.assertNotIn("background-image", view)
        # svg data (coming from ++plone++imio.smartweb.core/icons/vue-annuaire.svg)
        # we can't get it directly from the file because of all Plone filters
        # that changes the SVG content
        self.assertIn('<path d="M18 0H2V2H18V0ZM2 24H18V22H2V24ZM18', view)

    def test_icons_override(self):
        self.assertVocabularyLen("imio.smartweb.vocabulary.Icons", 49)
        portal_resources = getUtility(IResourceDirectory, name="persistent")
        portal_resources.makeDirectory("plone")
        portal_resources["plone"].makeDirectory("imio.smartweb.core")
        portal_resources["plone"]["imio.smartweb.core"].makeDirectory("icons")
        self.assertVocabularyLen("imio.smartweb.vocabulary.Icons", 0)
        portal_resources.writeFile(
            path="plone/imio.smartweb.core/icons/action-email.svg", data="test"
        )
        self.assertVocabularyLen("imio.smartweb.vocabulary.Icons", 1)



# -*- coding: utf-8 -*-
from Products.Five.browser import BrowserView
from imio.smartweb.core.contents import IPages
from imio.smartweb.core.contents.pages.procedure.utils import sign_url
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.api.portal import get_registry_record
from plone.formwidget.geolocation.vocabularies import _ as _geo
from zope.component import getMultiAdapter
from zope.i18n import translate

import json
import requests
import os


class UtilsView(BrowserView):
    """ """

    def is_previewable_content(self):
        if not IPages.providedBy(self.context):
            return False

        context_state = getMultiAdapter(
            (self.context, self.request), name="plone_context_state"
        )
        return context_state.is_view_template()

    def has_gdpr_text(self):
        return api.portal.get_registry_record(
            "imio.gdpr.interfaces.IGDPRSettings.is_text_ready", default=False
        )

    def map_configuration(self):
        """Returns global map configuration from registry"""
        map_layers = get_registry_record("geolocation.map_layers") or []
        config = {
            "fullscreencontrol": get_registry_record("geolocation.fullscreen_control"),
            "locatecontrol": get_registry_record("geolocation.locate_control"),
            "zoomcontrol": get_registry_record("geolocation.zoom_control"),
            "minimap": get_registry_record("geolocation.show_minimap"),
            "addmarker": get_registry_record("geolocation.show_add_marker"),
            "geosearch": get_registry_record("geolocation.show_geosearch"),
            "geosearch_provider": get_registry_record("geolocation.geosearch_provider"),
            "default_map_layer": get_registry_record("geolocation.default_map_layer"),
            "map_layers": [
                {"title": translate(_geo(layer), context=self.request), "id": layer}
                for layer in map_layers
            ],
            "latitude": get_registry_record("geolocation.default_latitude"),
            "longitude": get_registry_record("geolocation.default_longitude"),
        }
        return json.dumps(config)

    def is_eguichet_aware(self):
        self.request.response.setHeader("Content-Type", "application/json")
        url = api.portal.get_registry_record("smartweb.url_formdefs_api")
        key = api.portal.get_registry_record("smartweb.secret_key_api")
        orig = "ia.smartweb"
        if not url:
            return json.dumps(
                {"value": False, "text": _("smartweb.settings : No url define")}
            )
        if not key:
            return json.dumps(
                {"value": False, "text": _("smartweb.settings : No secret key define")}
            )
        query_full = sign_url(url, key, orig)
        response = requests.get(query_full, timeout=10)
        return json.dumps(
            {
                "status_code": response.status_code,
                "value": True,
                "text": _(response.reason),
            }
        )


def get_plausible_vars(self):
    env_plausible_url = os.getenv("SMARTWEB_PLAUSIBLE_URL", "")
    env_plausible_site = os.getenv("SMARTWEB_PLAUSIBLE_SITE", "")
    env_plausible_token = os.getenv("SMARTWEB_PLAUSIBLE_TOKEN", "")

    plausible_url = (
        env_plausible_url
        if (env_plausible_url and env_plausible_url != "")
        else api.portal.get_registry_record("smartweb.plausible_url")
    )
    plausible_site = (
        env_plausible_site
        if (env_plausible_site and env_plausible_site != "")
        else api.portal.get_registry_record("smartweb.plausible_site")
    )
    plausible_token = (
        env_plausible_token
        if (env_plausible_token and env_plausible_token != "")
        else api.portal.get_registry_record("smartweb.plausible_token")
    )
    if all([plausible_site, plausible_url, plausible_token]):
        plausible_vars = {
            "plausible_url": plausible_url,
            "plausible_site": plausible_site,
            "plausible_token": plausible_token,
        }
        return plausible_vars
    else:
        return None



# -*- coding: utf-8 -*-

from collective.instancebehavior.interfaces import IInstanceBehaviorAssignableContent
from imio.smartweb.core.contents import IPages
from imio.smartweb.core.contents import Pages
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.autoform.interfaces import IFormFieldProvider
from zope import schema
from zope.interface import implementer
from zope.interface import Invalid
from zope.interface import invariant
from zope.interface import provider


@provider(IFormFieldProvider)
class IProcedure(IPages):
    """ """

    procedure_ts = schema.Choice(
        vocabulary="imio.smartweb.vocabulary.PublikProcedures",
        title=_("E-Guichet procedure"),
        required=False,
        default=None,
    )

    procedure_url = schema.URI(title=_("Procedure url"), required=False)

    @invariant
    def required_procedure(data):
        # if data.procedure_ts is None and data.procedure_url is None:
        #     raise Invalid(_(u"Procedure field is required !"))

        if data.procedure_ts and data.procedure_url:
            raise Invalid(_("Only one procedure field can be filled !"))


@implementer(IProcedure, IInstanceBehaviorAssignableContent)
class Procedure(Pages):
    """Procedure class"""

    category_name = "procedure_category"



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents import IFolder
from imio.smartweb.core.contents.cropping import SmartwebCroppingProvider
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.autoform.interfaces import IFormFieldProvider
from plone.namedfile.field import NamedBlobImage
from plone.supermodel import model
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


class SubsiteCroppingProvider(SmartwebCroppingProvider):
    def get_scales(self, fieldname, request=None):
        if fieldname == "logo":
            # scale used for logo field
            return ["preview"]
        else:
            return super(SubsiteCroppingProvider, self).get_scales(fieldname, request)


@provider(IFormFieldProvider)
class IImioSmartwebSubsite(model.Schema):
    model.fieldset(
        "layout",
        label=_("Layout"),
        fields=["logo", "logo_display_mode"],
    )

    logo = NamedBlobImage(
        title=_("Logo"),
        description=_("Define a logo for subsite and children"),
        required=False,
    )

    logo_display_mode = schema.Choice(
        title=_("Logo display"),
        description=_("Choose your display mode"),
        source="imio.smartweb.vocabulary.SubsiteDisplayMode",
        required=True,
        default="title",
    )

    model.fieldset("settings", fields=["menu_depth"])
    menu_depth = schema.Int(
        title=_("Menu depth"),
        description=_("Define number of levels in menu navigation subsite"),
        required=True,
        default=2,
    )


@implementer(IImioSmartwebSubsite)
@adapter(IFolder)
class Subsite(object):
    """ """



# -*- coding: utf-8 -*-

from imio.smartweb.core.interfaces import IViewWithoutLeadImage
from plone import api
from Products.Five import BrowserView
from zope.interface import implementer


@implementer(IViewWithoutLeadImage)
class BaseRestView(BrowserView):
    @property
    def batch_size(self):
        return self.context.nb_results

    @property
    def local_query_url(self):
        base_url = self.context.absolute_url()
        return f"{base_url}/@results"

    @property
    def local_filters_query_url(self):
        base_url = self.context.absolute_url()
        return f"{base_url}/@results-filters"

    def get_current_language(self):
        return api.portal.get_current_language()[:2]



# -*- coding: utf-8 -*-

from datetime import datetime
from imio.smartweb.core.config import DIRECTORY_URL
from imio.smartweb.core.config import EVENTS_URL
from imio.smartweb.core.config import NEWS_URL
from plone import api
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.memoize import ram
from plone.restapi.search.handler import SearchHandler
from plone.restapi.search.utils import unflatten_dotted_dict
from plone.restapi.services import Service
from time import time


def _cache_key(func, navigation_root):
    uid_method = getattr(navigation_root, "UID", None)
    uid = uid_method and uid_method() or "PloneSite"
    return (uid, time() // (5 * 60))


@ram.cache(_cache_key)
def get_news_views(navigation_root):
    with api.env.adopt_user(username="admin"):
        brains = api.content.find(
            context=navigation_root, portal_type="imio.smartweb.NewsView"
        )
        # Can be improved by using index
        return {b.getObject().selected_news_folder: b.getURL() for b in brains}


@ram.cache(_cache_key)
def get_events_views(navigation_root):
    with api.env.adopt_user(username="admin"):
        brains = api.content.find(
            context=navigation_root, portal_type="imio.smartweb.EventsView"
        )
        # Can be improved by using index
        return {b.getObject().selected_agenda: b.getURL() for b in brains}


def get_navigation_root(context):
    """Return given context navigation root"""
    if INavigationRoot.providedBy(context):
        return context
    else:
        for item in context.aq_chain:
            if INavigationRoot.providedBy(item):
                return item


def get_default_view_url(view_type):
    record = api.portal.get_registry_record(
        "smartweb.default_{0}_view".format(view_type), default=None
    )
    if not record:
        return ""
    with api.env.adopt_user(username="admin"):
        return api.content.get(UID=record).absolute_url()


def get_views_mapping(navigation_root):
    news = get_news_views(navigation_root)
    news["default"] = get_default_view_url("news")

    events = get_events_views(navigation_root)
    events["default"] = get_default_view_url("events")

    # Directory is a special usecase, since there is no selected entity
    # the global entity must be used
    default_directory_view = get_default_view_url("directory")
    directory_entity_uid = api.portal.get_registry_record(
        "smartweb.directory_entity_uid"
    )
    directory = {
        directory_entity_uid: default_directory_view,
        "default": default_directory_view,
    }

    mapping = {
        "imio.news.NewsItem": news,
        "imio.events.Event": events,
        "imio.directory.Contact": directory,
    }
    return mapping


class SearchGet(Service):
    def reply(self):
        query = self.request.form.copy()
        query = unflatten_dotted_dict(query)
        return ExtendedSearchHandler(self.context, self.request).search(query)


class ExtendedSearchHandler(SearchHandler):
    def search(self, query):
        if "_core" not in query and "use_site_search_settings" not in query:
            # Use site search settings by default only for current site search
            query["use_site_search_settings"] = True
        if "use_solr" not in query:
            query["use_solr"] = True  # enforce use of SolR by default
        if "_core" in query:
            language = api.portal.get_current_language(context=self.context)
            parameters = self._core_query(query["_core"], language=language)
            del query["_core"]
            if parameters:
                if "metadata_fields" in query:
                    parameters["metadata_fields"] = self._update_metadata_fields(
                        query["metadata_fields"],
                        parameters["metadata_fields"],
                    )
                self.request.form["metadata_fields"] = parameters["metadata_fields"]
                query.update(parameters)
                if language != "fr" and "SearchableText" in query:
                    basesearch = query["SearchableText"]
                    query[f"SearchableText_{language}"] = {
                        "query": [basesearch, f"{basesearch}*"],
                        "operator": "or",
                    }
                    del query["SearchableText"]
        result = super(ExtendedSearchHandler, self).search(query)
        if "core" in query:
            return self._adapt_result(result, query["core"], language=language)
        return result

    def _get_source_url(self, path, core):
        base_urls = {
            "directory": DIRECTORY_URL,
            "news": NEWS_URL,
            "events": EVENTS_URL,
        }
        return "{0}/{1}".format(base_urls.get(core, ""), "/".join(path.split("/")[2:]))

    def _adapt_result(self, result, core, language=None):
        """Transform result"""
        mapping = get_views_mapping(self._navigation_root)
        result["items"] = [
            self._adapt_result_values(i, mapping, core, language=language)
            for i in result["items"]
        ]
        return result

    def _adapt_result_values(self, item, mapping, core, language=None):
        """Ensure that url to external objects are adapted based on react views"""
        if item["@type"] not in mapping:
            return item
        if language is not None and language != "fr":
            for field in ("description", "title"):
                fname = f"{field}_{language}"
                if fname in item:
                    item[field] = item[fname]
                    del item[fname]
        type_mapping = mapping[item["@type"]]
        base_url = type_mapping.get(item["container_uid"], type_mapping["default"])
        item["_url"] = "{base}#/content?u={item_uid}".format(
            base=base_url,
            item_uid=item["UID"],
        )
        item["_source_url"] = self._get_source_url(item["path_string"], core)
        return item

    @property
    def _navigation_root(self):
        return get_navigation_root(self.context)

    def _update_metadata_fields(self, original, new):
        if isinstance(original, str):
            original = [original]
        return [*original, *[e for e in new if e not in original]]

    def _core_query(self, core, language=None):
        """Return core specific query parameters"""
        mapping = get_views_mapping(self._navigation_root)
        parameters = {
            "news": {
                "portal_type": ["imio.news.NewsItem"],
                "metadata_fields": [
                    "id",
                    "UID",
                    "container_uid",
                    "has_leadimage",
                    "path_string",
                ],
                "selected_news_folders": {
                    "query": [
                        k
                        for k in mapping["imio.news.NewsItem"].keys()
                        if k != "default"
                    ],
                    "operator": "or",
                },
                "expires": {"query": datetime.now(), "range": "min"},
                "path": "",
            },
            "events": {
                "portal_type": ["imio.events.Event"],
                "metadata_fields": [
                    "id",
                    "UID",
                    "container_uid",
                    "has_leadimage",
                    "path_string",
                ],
                "selected_agendas": {
                    "query": [
                        k for k in mapping["imio.events.Event"].keys() if k != "default"
                    ],
                    "operator": "or",
                },
                "event_dates": {"query": datetime.now().date(), "range": "min"},
                "expires": {"query": datetime.now(), "range": "min"},
                "path": "",
            },
            "directory": {
                "portal_type": ["imio.directory.Contact"],
                "metadata_fields": [
                    "id",
                    "UID",
                    "container_uid",
                    "has_leadimage",
                    "path_string",
                ],
                "selected_entities": {
                    "query": [
                        k
                        for k in mapping["imio.directory.Contact"].keys()
                        if k != "default"
                    ],
                    "operator": "or",
                },
                "expires": {"query": datetime.now(), "range": "min"},
                "path": "",
            },
        }.get(core)

        required_fields = {
            "news": "selected_news_folders",
            "events": "selected_agendas",
            "directory": "selected_entities",
        }

        if parameters:
            parameters["core"] = api.portal.get_registry_record(
                "smartweb.{0}_solr_core".format(core)
            )
            required_field = required_fields[core]
            if not parameters.get(required_field)["query"]:
                # This avoid having unwanted results if no view was created
                parameters[required_field] = ["None"]
            if language is not None and language != "fr":
                parameters["metadata_fields"].extend(
                    [f"title_{language}", f"description_{language}"]
                )
                parameters[f"translated_in_{language}"] = True
        return parameters



# -*- coding: utf-8 -*-

from imio.smartweb.common.adapters import BaseCroppingProvider
from plone.app.imagecropping.dx import CroppingUtilsDexterity
from plone.app.imagecropping.interfaces import IImageCroppingUtils
from plone.namedfile.interfaces import IImage
from plone.namedfile.interfaces import IImageScaleTraversable
from zope.component import adapter
from zope.interface import implementer


UNCROPPABLE_FIELDS = ["banner"]


class SmartwebCroppingProvider(BaseCroppingProvider):
    def get_scales(self, fieldname, request=None):
        """Define default cropping scales for all common fields"""
        if fieldname in ["banner", "background_image"]:
            # scale used for banner fields or background_image fields
            return []
        elif fieldname == "image":
            # scales used for lead_image fields
            return ["liste", "vignette", "slide"]
        else:
            return super(SmartwebCroppingProvider, self).get_scales(fieldname, request)


@implementer(IImageCroppingUtils)
@adapter(IImageScaleTraversable)
class SmartwebCroppingUtilsDexterity(CroppingUtilsDexterity):
    def _image_field_values(self):
        """Remove banner field from cropping editor"""
        for fieldname, field in self._all_fields():
            value = getattr(self.context, fieldname, None)
            if (
                value
                and IImage.providedBy(value)
                and fieldname not in UNCROPPABLE_FIELDS
            ):
                yield (fieldname, value)



# -*- coding: utf-8 -*-

from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_INTEGRATION_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.viewlets.herobanner import HeroBannerViewlet
from plone import api
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from zope.component import getMultiAdapter


class TestHeroBanner(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="Folder",
            id="folder",
        )

    def test_add_herobanner_to_plonesite(self):
        view = getMultiAdapter((self.portal, self.request), name="herobanner_settings")
        self.assertTrue(view.available())
        viewlet = HeroBannerViewlet(self.portal, self.request, None, None)
        viewlet.update()
        self.assertFalse(viewlet.available())
        view.add_herobanner()
        herobanners = self.portal.listFolderContents(
            contentFilter={"portal_type": "imio.smartweb.HeroBanner"}
        )
        self.assertEqual(len(herobanners), 1)
        self.assertFalse(view.available())
        viewlet = HeroBannerViewlet(self.portal, self.request, None, None)
        viewlet.update()
        self.assertTrue(viewlet.available())
        self.assertEqual(viewlet.herobanner, herobanners[0])

        default_page = api.content.create(
            container=self.portal,
            type="imio.smartweb.PortalPage",
            id="portal-page",
        )
        self.portal.setDefaultPage("portal-page")
        viewlet = HeroBannerViewlet(default_page, self.request, None, None)
        viewlet.update()
        self.assertEqual(viewlet.herobanner, herobanners[0])
        view = getMultiAdapter((default_page, self.request), name="herobanner_settings")
        self.assertFalse(view.available())

        api.content.delete(herobanners[0])
        self.assertTrue(view.available())

        view = getMultiAdapter((default_page, self.request), name="herobanner_settings")
        view.add_herobanner()
        herobanners = self.portal.listFolderContents(
            contentFilter={"portal_type": "imio.smartweb.HeroBanner"}
        )
        self.assertEqual(len(herobanners), 1)
        herobanners = default_page.listFolderContents(
            contentFilter={"portal_type": "imio.smartweb.HeroBanner"}
        )
        self.assertEqual(len(herobanners), 0)

    def test_add_herobanner_to_folder(self):
        view = getMultiAdapter((self.folder, self.request), name="herobanner_settings")
        self.assertFalse(view.available())
        view.add_herobanner()
        herobanners = self.portal.listFolderContents(
            contentFilter={"portal_type": "imio.smartweb.HeroBanner"}
        )
        self.assertEqual(len(herobanners), 0)

    def test_add_herobanner_to_minisite(self):
        herobanner_view = getMultiAdapter(
            (self.folder, self.request), name="herobanner_settings"
        )
        self.assertFalse(herobanner_view.available())
        minisite_view = getMultiAdapter(
            (self.folder, self.request), name="minisite_settings"
        )
        minisite_view.enable()
        self.assertTrue(herobanner_view.available())
        herobanner_view.add_herobanner()
        herobanners = self.folder.listFolderContents(
            contentFilter={"portal_type": "imio.smartweb.HeroBanner"}
        )
        self.assertEqual(len(herobanners), 1)
        self.assertFalse(herobanner_view.available())

        default_page = api.content.create(
            container=self.folder,
            type="imio.smartweb.PortalPage",
            id="portal-page",
        )
        self.folder.set_default_item(new_default_item=default_page)
        viewlet = HeroBannerViewlet(default_page, self.request, None, None)
        viewlet.update()
        self.assertEqual(viewlet.herobanner, herobanners[0])
        herobanner_view = getMultiAdapter(
            (default_page, self.request), name="herobanner_settings"
        )
        self.assertFalse(herobanner_view.available())

        api.content.delete(herobanners[0])
        self.assertTrue(herobanner_view.available())

    def test_exclude_from_parent_listing(self):
        view = getMultiAdapter((self.portal, self.request), name="herobanner_settings")
        view.add_herobanner()
        herobanners = self.portal.listFolderContents(
            contentFilter={"portal_type": "imio.smartweb.HeroBanner"}
        )
        self.assertTrue(herobanners[0].exclude_from_parent_listing)

    def test_sections_render(self):
        herobanner_view = getMultiAdapter(
            (self.portal, self.request), name="herobanner_settings"
        )
        herobanner_view.add_herobanner()
        herobanner = getattr(self.portal, "herobanner")
        slide = api.content.create(
            container=herobanner,
            type="imio.smartweb.SectionSlide",
            title="Section slide",
        )
        slide.show_title_and_description = True
        viewlet = HeroBannerViewlet(self.portal, self.request, None, None)
        viewlet.update()
        sections = viewlet.sections
        self.assertEqual(len(sections), 1)
        render = "\n".join(
            [
                getMultiAdapter(
                    (section, self.request), name="full_view_item_without_edit"
                )()
                for section in sections
            ]
        )
        self.assertIn("Section slide", render)
        self.assertNotIn("<h2", render)
        self.assertNotIn('class="manage-section"', render)
        self.assertNotIn("section-container", render)



# -*- coding: utf-8 -*-

from imio.smartweb.core.behaviors.subsite import IImioSmartwebSubsite
from imio.smartweb.core.contents import IFolder
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.app.layout.navigation.interfaces import INavigationRoot
from Products.CMFPlone.interfaces.constrains import DISABLED
from Products.CMFPlone.interfaces.constrains import ISelectableConstrainTypes
from Products.Five.browser import BrowserView


class FooterSettings(BrowserView):
    """Footer settings"""

    def add_footer(self):
        if not self.available():
            api.portal.show_message(
                _("Footer cannot be added here"), self.request, type="error"
            )
            self.request.response.redirect(self.context.absolute_url())
            return ""
        if IFolder.providedBy(self.context):
            container = ISelectableConstrainTypes(self.context)
            constrain_types_mode = container.getConstrainTypesMode()
            container.setConstrainTypesMode(DISABLED)
        pt = api.portal.get_tool("portal_types")
        portal_type = self.context.portal_type
        allowed_content_types = pt.getTypeInfo(portal_type).allowed_content_types
        allowed_content_types = list(allowed_content_types)
        allowed_content_types.append("imio.smartweb.Footer")
        pt.getTypeInfo(portal_type).allowed_content_types = tuple(allowed_content_types)
        footer = api.content.create(
            title="Footer",
            id="footer",
            container=self.context,
            type="imio.smartweb.Footer",
        )
        footer.exclude_from_parent_listing = True
        footer.reindexObject(idxs=["exclude_from_parent_listing"])
        allowed_content_types.remove("imio.smartweb.Footer")
        pt.getTypeInfo(portal_type).allowed_content_types = tuple(allowed_content_types)
        if IFolder.providedBy(self.context):
            container.setConstrainTypesMode(constrain_types_mode)
        api.portal.show_message(_("Footer has been successfully added"), self.request)
        self.request.response.redirect(footer.absolute_url())

    def available(self):
        if not INavigationRoot.providedBy(
            self.context
        ) and not IImioSmartwebSubsite.providedBy(self.context):
            return False
        footers = self.context.listFolderContents(
            contentFilter={"portal_type": "imio.smartweb.Footer"}
        )
        return len(footers) == 0



# -*- coding: utf-8 -*-

from imio.smartweb.core.utils import get_category
from plone.app.layout.viewlets import common


class CategoryViewlet(common.ViewletBase):
    def available(self):
        return self.get_category() is not None

    def get_category(self):
        return get_category(self.context)



# -*- coding: utf-8 -*-

from Acquisition import aq_inner
from imio.smartweb.core.utils import get_scale_url
from imio.smartweb.core.interfaces import IViewWithoutLeadImage
from imio.smartweb.locales import SmartwebMessageFactory as _
from Products.CMFPlone.resources import add_bundle_on_request
from plone import api
from plone.app.content.browser.contents.rearrange import OrderContentsBaseAction
from plone.app.content.utils import json_loads
from plone.app.contenttypes.browser.folder import FolderView
from plone.app.contenttypes.browser.full_view import FullViewItem as BaseFullViewItem
from Products.CMFPlone import utils
from Products.CMFPlone.browser.navigation import PhysicalNavigationBreadcrumbs
from zope.component import getMultiAdapter
from zope.interface import implementer


@implementer(IViewWithoutLeadImage)
class PagesView(FolderView):
    """Pages view"""

    b_size = 40

    def __call__(self):
        # get gallery sections and sections with gallery
        galleries_sections = self.context.listFolderContents(
            contentFilter={
                "portal_type": [
                    "imio.smartweb.SectionGallery",
                    "imio.smartweb.SectionContact",
                ]
            }
        )
        if len(galleries_sections) > 0:
            add_bundle_on_request(self.request, "spotlight")
        for index, gs in enumerate(galleries_sections):
            if (
                gs.portal_type == "imio.smartweb.SectionContact"
                and gs.gallery_mode != "gallery"
            ):
                galleries_sections.pop(index)
        if len(galleries_sections) > 0:
            add_bundle_on_request(self.request, "flexbin")
        return self.index()

    def results(self, **kwargs):
        """
        Gets results for folder listings without taking care of friendly_types
        """
        # Extra filter
        kwargs.update(self.request.get("contentFilter", {}))
        kwargs.setdefault("batch", True)
        kwargs.setdefault("b_size", self.b_size)
        kwargs.setdefault("b_start", self.b_start)
        kwargs.setdefault("orphan", 1)

        listing = aq_inner(self.context).restrictedTraverse("@@folderListing", None)
        results = listing(**kwargs)
        return results

    @property
    def no_items_message(self):
        return _("There is no section on this page.")

    def get_class(self, obj):
        section_type = obj.portal_type.split(".")[-1]
        return " ".join(
            filter(
                None,
                [
                    section_type.lower(),
                    obj.css_class or "",
                    obj.bootstrap_css_class or "",
                    self.background_style(obj) and "with-background" or "",
                ],
            )
        )

    def background_style(self, obj):
        if not obj.background_image:
            return ""
        scale_url = get_scale_url(obj, self.request, "background_image", "large")
        css_bg_image = f"background-image:url({scale_url});"
        css_bg_size = "background-size:cover;"
        return " ".join([css_bg_image, css_bg_size])


class PagesFullViewItem(BaseFullViewItem):
    """Page view item"""

    def __init__(self, context, request):
        super(PagesFullViewItem, self).__init__(context, request)
        self.can_edit_sections = api.user.has_permission(
            "Modify portal content", obj=context
        )


class PagesFullViewItemWithoutEdit(BaseFullViewItem):
    """Page view item without sections edit tools"""

    can_edit_sections = False


class PagesOrderingView(OrderContentsBaseAction):
    """Page sections ordering view"""

    def __call__(self):
        self.protect()
        form = self.request.form
        id = form.get("id")
        delta = int(form.get("delta"))
        ordering = self.getOrdering()
        ordered_ids = json_loads(form.get("orderedSectionsIds", "null"))
        if ordered_ids:
            position_id = [(ordering.getObjectPosition(i), i) for i in ordered_ids]
            position_id.sort()
            if ordered_ids != [i for position, i in position_id]:
                return
        ordering.moveObjectsByDelta([id], delta, ordered_ids)


class DefaultPagesBreadcrumbs(PhysicalNavigationBreadcrumbs):
    def breadcrumbs(self):
        """
        Change breadcrumbs behaviour for default pages :
            - anonymous user does not see the page
            - connected user sees the page
        """
        if not api.user.is_anonymous():
            return super(DefaultPagesBreadcrumbs, self).breadcrumbs()
        context = aq_inner(self.context)
        request = self.request
        container = utils.parent(context)
        view = getMultiAdapter((container, request), name="breadcrumbs_view")
        return tuple(view.breadcrumbs())



# -*- coding: utf-8 -*-

from imio.smartweb.common.browser.forms import CustomAddForm
from imio.smartweb.core.browser.forms import SmartwebCustomEditForm
from plone.dexterity.browser.add import DefaultAddView
from plone.z3cform import layout
from z3c.form.interfaces import HIDDEN_MODE


class ContactCustomAddForm(CustomAddForm):
    portal_type = "imio.smartweb.SectionContact"

    def update(self):
        super(ContactCustomAddForm, self).update()
        # We hide hide_title field so no one can change the value for contact
        # and set True value (single checkbox)
        for group in self.groups:
            if group.__name__ == "layout":
                group.widgets["hide_title"].mode = HIDDEN_MODE
                group.widgets["hide_title"].value = ["selected"]


class ContactCustomAddView(DefaultAddView):
    form = ContactCustomAddForm


class ContactCustomEditForm(SmartwebCustomEditForm):
    def update(self):
        super(ContactCustomEditForm, self).update()
        # We hide hide_title field so no one can change the value for contact
        # and set True value (single checkbox)
        for group in self.groups:
            if group.__name__ == "layout":
                group.widgets["hide_title"].mode = HIDDEN_MODE
                group.widgets["hide_title"].value = ["selected"]


ContactCustomEditView = layout.wrap_form(ContactCustomEditForm)



# -*- coding: utf-8 -*-

from bs4 import BeautifulSoup
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_INTEGRATION_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.tests.utils import make_named_image
from imio.smartweb.core.viewlets.logo import LogoViewlet
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from plone.formwidget.namedfile.converter import b64encode_file
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.interfaces import ISiteSchema
from zope.component import getUtility


class TestLogo(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_portal_logo(self):
        viewlet = LogoViewlet(self.portal, self.request, None, None)
        viewlet.update()
        self.assertFalse(viewlet.show_title)
        self.assertTrue(viewlet.show_logo)
        self.assertFalse(viewlet.is_svg)
        self.assertEqual(viewlet.navigation_root_url, "http://nohost/plone")
        html = viewlet.render()
        soup = BeautifulSoup(html)
        img = soup.find("img")
        self.assertEqual(
            img.get("src"), "http://nohost/plone/++resource++plone-logo.svg"
        )

        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSchema, prefix="plone")
        encoded_data = b64encode_file(**make_named_image("plone.svg"))
        settings.site_logo = encoded_data
        viewlet.update()
        self.assertTrue(viewlet.show_logo)
        self.assertTrue(viewlet.is_svg)
        self.assertIn(b"<svg", viewlet.svg_data)
        html = viewlet.render()
        soup = BeautifulSoup(html)
        img = soup.find("img")
        self.assertIsNone(img)

        encoded_data = b64encode_file(**make_named_image())
        settings.site_logo = encoded_data
        viewlet = LogoViewlet(self.portal, self.request, None, None)
        viewlet.update()
        self.assertTrue(viewlet.show_logo)
        self.assertFalse(viewlet.is_svg)
        html = viewlet.render()
        soup = BeautifulSoup(html)
        img = soup.find("img")
        self.assertEqual(img.get("src"), "http://nohost/plone/@@site-logo/plone.png")



# -*- coding: utf-8 -*-

from collective.faceted.map.browser.map import MapView
from imio.smartweb.core.utils import get_scale_url
from imio.smartweb.core.browser.faceted.views import FolderView
from Products.Five import BrowserView


class FacetedMapView(FolderView, MapView):
    """Faceted map view"""

    def get_scale_url(self, item):
        request = self.request
        return get_scale_url(item, request, "image", "preview")


class FacetedGeoJSONPopup(BrowserView):
    def popup(self, brain):
        url = brain.getURL()
        title = brain.Title
        description = brain.Description
        if brain.has_leadimage:
            img_url = get_scale_url(brain, self.request, "image", "mini")
            return f"""<a href="{url}" title="{title}">
                         <img src="{img_url}" alt="{title}" />
                         <div>
                           <span class="popup_title">{title}</span>
                           <span class="popup_description">{description}</span>
                         </div>
                       </a>"""
        else:
            return f"""<a href="{url}" title="{title}">
                         <div>
                           <span class="popup_title">{title}</span>
                           <span class="popup_description">{description}</span>
                         </div>
                       </a>"""



# -*- coding: utf-8 -*-

from collective.taxonomy.interfaces import ITaxonomy
from datetime import date
from imio.smartweb.common.indexers import breadcrumb
from imio.smartweb.common.utils import get_vocabulary
from imio.smartweb.core.config import DIRECTORY_URL, EVENTS_URL, NEWS_URL
from imio.smartweb.core.contents import IPages
from imio.smartweb.core.contents.pages.procedure.utils import sign_url
from imio.smartweb.core.interfaces import ISmartwebIcon
from imio.smartweb.core.utils import concat_voca_term
from imio.smartweb.core.utils import concat_voca_title
from imio.smartweb.core.utils import get_categories
from imio.smartweb.core.utils import get_json
from imio.smartweb.core.utils import get_wca_token
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.app.contenttypes.interfaces import ICollection
from plone.dexterity.interfaces import IDexterityContent
from plone.memoize import ram
from plone.registry.interfaces import IRegistry
from time import time
from zExceptions import NotFound
from zope.component import getUtility
from zope.component.hooks import getSite
from zope.i18n import translate
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary

import json
import os
import requests


class IconsVocabularyFactory:
    def __call__(self, context=None):
        site = getSite()
        language = api.portal.get_current_language(context=context)
        icon_prefix = "smartweb.icon."
        registry = getUtility(IRegistry)
        icons = []
        for record in registry.records.values():
            if record.interfaceName != ISmartwebIcon.__identifier__:
                continue
            name = record.__name__.replace(icon_prefix, "")
            icon_url = record.value
            try:
                site.unrestrictedTraverse(icon_url)
            except NotFound:
                # if icons folder is overridden through portal_resources, some
                # icons may be missing. Ignore them.
                continue
            icons.append(
                {
                    "title": translate(record.field.title, target_language=language),
                    "name": name,
                }
            )
        icons.sort(key=lambda k: k["title"])
        terms = [
            SimpleTerm(value=i["name"], token=i["name"], title=i["title"])
            for i in icons
        ]
        return SimpleVocabulary(terms)


IconsVocabulary = IconsVocabularyFactory()


class RemoteProceduresVocabularyFactory:
    def __call__(self, context=None):
        # sample : "https://olln-formulaires.guichet-citoyen.be/api/formdefs/"
        url = api.portal.get_registry_record("smartweb.url_formdefs_api")
        # sample : "568DGess2x8j8twv7x2Y2MApjn789xfG7jM27r399q4xSD27Jz"
        key = api.portal.get_registry_record("smartweb.secret_key_api")
        orig = "ia.smartweb"
        if not url:
            return SimpleVocabulary([])
        query_full = sign_url(url, key, orig)
        try:
            response = requests.get(query_full)
        except Exception:
            return SimpleVocabulary([])

        if response.status_code != 200:
            return SimpleVocabulary([])

        json_procedures = json.loads(response.text)
        return SimpleVocabulary(
            [
                SimpleTerm(value=elem["url"], title=elem["title"])
                for elem in json_procedures.get("data", [])
            ]
        )


RemoteProceduresVocabulary = RemoteProceduresVocabularyFactory()


class CurrentFolderPagesVocabularyFactory:
    def __call__(self, context=None):
        brains = api.content.find(
            context=context,
            depth=1,
            object_provides=[IPages, ICollection],
            sort_on="sortable_title",
        )
        brains = [
            b
            for b in brains
            if b.portal_type not in ["imio.smartweb.Footer", "imio.smartweb.HeroBanner"]
        ]
        terms = [SimpleTerm(value=b.UID, token=b.UID, title=b.Title) for b in brains]
        return SimpleVocabulary(terms)


CurrentFolderPagesVocabulary = CurrentFolderPagesVocabularyFactory()


class BootstrapCSSVocabularyFactory:
    def __call__(self, context=None):
        bootstrap_css = [
            ("col-sm-3", _("Quarter of width")),
            ("col-sm-4", _("Third of width")),
            ("col-sm-6", _("Half of width")),
            ("col-sm-8", _("Two third of width")),
            ("col-sm-9", _("Three quarter of width")),
            ("col-sm-12", _("Full width")),
        ]
        terms = [SimpleTerm(value=t[0], token=t[0], title=t[1]) for t in bootstrap_css]
        return SimpleVocabulary(terms)


BootstrapCSSVocabulary = BootstrapCSSVocabularyFactory()


class SubsiteDisplayModeVocabularyFactory:
    def __call__(self, context=None):
        display_mode = [
            ("title", _("Title")),
            ("logo", _("Logo")),
            ("logo_title", _("Logo and title")),
        ]
        terms = [SimpleTerm(value=t[0], token=t[0], title=t[1]) for t in display_mode]
        return SimpleVocabulary(terms)


SubsiteDisplayModeVocabulary = SubsiteDisplayModeVocabularyFactory()


class ContactBlocksVocabularyFactory:
    def __call__(self, context=None):
        values = [
            ("logo", _("Logo")),
            ("leadimage", _("Lead Image")),
            ("titles", _("Title and Subtitle")),
            ("contact_informations", _("Contact informations")),
            ("address", _("Address")),
            ("itinerary", _("Itinerary")),
            ("schedule", _("Schedule")),
            ("map", _("Map")),
            ("description", _("Description")),
            ("gallery", _("Gallery")),
        ]
        terms = [
            SimpleVocabulary.createTerm(value[0], value[0], value[1])
            for value in values
        ]
        return SimpleVocabulary(terms)


ContactBlocksVocabulary = ContactBlocksVocabularyFactory()


class RemoteContactsVocabularyFactory:
    def __call__(self, context=None):
        entity_uid = api.portal.get_registry_record("smartweb.directory_entity_uid")
        params = [
            "portal_type=imio.directory.Contact",
            "selected_entities={}".format(entity_uid),
            "sort_on=breadcrumb",
            "b_size=1000000",
            "metadata_fields=UID",
            "metadata_fields=breadcrumb",
        ]
        url = "{}/@search?{}".format(DIRECTORY_URL, "&".join(params))
        json_contacts = get_json(url)
        if json_contacts is None or len(json_contacts.get("items", [])) == 0:
            return SimpleVocabulary([])
        return SimpleVocabulary(
            [
                SimpleTerm(value=elem["UID"], title=elem["breadcrumb"])
                for elem in json_contacts.get("items")
            ]
        )


RemoteContactsVocabulary = RemoteContactsVocabularyFactory()


def get_entities_vocabulary(portal_type, base_url):
    params = [
        "portal_type={}".format(portal_type),
        "sort_on=sortable_title",
        "b_size=1000000",
        "metadata_fields=UID",
    ]
    url = "{}/@search?{}".format(base_url, "&".join(params))
    json_entities = get_json(url)
    if json_entities is None or len(json_entities.get("items", [])) == 0:
        return SimpleVocabulary([])
    return SimpleVocabulary(
        [
            SimpleTerm(value=elem["UID"], title=elem["title"])
            for elem in json_entities.get("items")
        ]
    )


class RemoteDirectoryEntitiesVocabularyFactory:
    def __call__(self, context=None):
        return get_entities_vocabulary("imio.directory.Entity", DIRECTORY_URL)


RemoteDirectoryEntitiesVocabulary = RemoteDirectoryEntitiesVocabularyFactory()


class RemoteEventsEntitiesVocabularyFactory:
    def __call__(self, context=None):
        return get_entities_vocabulary("imio.events.Entity", EVENTS_URL)


RemoteEventsEntitiesVocabulary = RemoteEventsEntitiesVocabularyFactory()


class RemoteNewsEntitiesVocabularyFactory:
    def __call__(self, context=None):
        return get_entities_vocabulary("imio.news.Entity", NEWS_URL)


RemoteNewsEntitiesVocabulary = RemoteNewsEntitiesVocabularyFactory()


def content_container_vocabulary(entity_uid, portal_type, base_url):
    """Gets containers of events / news"""
    url = "{}/@search?UID={}".format(base_url, entity_uid)
    entity_json = get_json(url)
    entity_url = entity_json.get("items")[0].get("@id")
    params = [
        "portal_type={}".format(portal_type),
        "sort_on=sortable_title",
        "b_size=1000000",
        "metadata_fields=UID",
    ]
    url = "{}/@search?{}".format(entity_url, "&".join(params))
    json_folders = get_json(url)
    if json_folders is None or len(json_folders.get("items", [])) == 0:
        return SimpleVocabulary([])
    return SimpleVocabulary(
        [
            SimpleTerm(value=elem["UID"], token=elem["UID"], title=elem["title"])
            for elem in json_folders.get("items")
        ]
    )


class RemoteAgendasVocabularyFactory:
    def __call__(self, context=None):
        entity_uid = api.portal.get_registry_record("smartweb.events_entity_uid")
        return content_container_vocabulary(
            entity_uid, "imio.events.Agenda", EVENTS_URL
        )


RemoteAgendasVocabulary = RemoteAgendasVocabularyFactory()


class RemoteNewsFoldersVocabularyFactory:
    def __call__(self, context=None):
        entity_uid = api.portal.get_registry_record("smartweb.news_entity_uid")
        return content_container_vocabulary(
            entity_uid, "imio.news.NewsFolder", NEWS_URL
        )


RemoteNewsFoldersVocabulary = RemoteNewsFoldersVocabularyFactory()


class AlignmentVocabularyFactory:
    def __call__(self, context=None):
        alignment = [
            ("left", _("Left")),
            ("right", _("Right")),
            ("bottom", _("Bottom")),
            ("top", _("Top")),
        ]
        terms = [SimpleTerm(value=t[0], token=t[0], title=t[1]) for t in alignment]
        return SimpleVocabulary(terms)


AlignmentVocabulary = AlignmentVocabularyFactory()


class ImageSizeVocabularyFactory:
    def __call__(self, context=None):
        image_size = [
            ("extralarge", _("Full width")),
            ("large", _("Half page")),
            ("medium", _("Third page")),
        ]
        terms = [SimpleTerm(value=t[0], token=t[0], title=t[1]) for t in image_size]
        return SimpleVocabulary(terms)


ImageSizeVocabulary = ImageSizeVocabularyFactory()


class DirectoryViewsVocabularyFactory(object):
    def __call__(self, context=None):
        if IDexterityContent.providedBy(context):
            context = api.portal.get_navigation_root(context)
        else:
            context = api.portal.get()
        brains = api.content.find(
            context=context, portal_type="imio.smartweb.DirectoryView"
        )
        terms = [
            SimpleTerm(value=b.UID, token=b.UID, title=breadcrumb(b.getObject()))
            for b in brains
        ]
        return SimpleVocabulary(terms)


DirectoryViewsVocabulary = DirectoryViewsVocabularyFactory()


class EventsViewsVocabularyFactory(object):
    def __call__(self, context=None):
        if IDexterityContent.providedBy(context):
            context = api.portal.get_navigation_root(context)
        else:
            context = api.portal.get()
        brains = api.content.find(
            context=context, portal_type="imio.smartweb.EventsView"
        )
        terms = [
            SimpleTerm(value=b.UID, token=b.UID, title=breadcrumb(b.getObject()))
            for b in brains
        ]
        return SimpleVocabulary(terms)


EventsViewsVocabulary = EventsViewsVocabularyFactory()


class NewsViewsVocabularyFactory(object):
    def __call__(self, context=None):
        if IDexterityContent.providedBy(context):
            context = api.portal.get_navigation_root(context)
        else:
            context = api.portal.get()
        brains = api.content.find(context=context, portal_type="imio.smartweb.NewsView")
        terms = [
            SimpleTerm(value=b.UID, token=b.UID, title=breadcrumb(b.getObject()))
            for b in brains
        ]
        return SimpleVocabulary(terms)


NewsViewsVocabulary = NewsViewsVocabularyFactory()


class CategoryAndTopicsVocabularyFactory:
    def __call__(self, context=None):
        categories_taxo = get_categories()
        language = api.portal.get_current_language(context=context)
        categories_voca = categories_taxo.makeVocabulary(language).inv_data

        topics_voca_factory = getUtility(
            IVocabularyFactory, "imio.smartweb.vocabulary.Topics"
        )
        topics_voca = topics_voca_factory(context)

        terms = []

        for cat in categories_voca:
            for topic in topics_voca:
                term = SimpleTerm(
                    value=concat_voca_term(cat, topic.value),
                    token=concat_voca_term(cat, topic.token),
                    title=concat_voca_title(
                        categories_voca[cat],
                        translate(topic.title, target_language=language),
                    ),
                )
                terms.append(term)

            terms.append(
                SimpleTerm(
                    value=cat,
                    token=cat,
                    title=categories_voca[cat],
                )
            )
        return SimpleVocabulary(terms)


CategoryAndTopicsVocabulary = CategoryAndTopicsVocabularyFactory()


class FilteredCategoryAndTopicsVocabularyFactory:
    def __call__(self, context=None):
        voca_registry = api.portal.get_registry_record(
            name="smartweb.category_and_topics_vocabulary"
        )
        terms = []
        if voca_registry is not None:
            terms = [
                SimpleTerm(value=r, token=r, title=voca_registry[r])
                for r in voca_registry
            ]

        return SimpleVocabulary(terms)


FilteredCategoryAndTopicsVocabulary = FilteredCategoryAndTopicsVocabularyFactory()


class DirectoryCategoriesVocabularyFactory:
    def __call__(self, context=None):
        client_id = os.environ.get("RESTAPI_DIRECTORY_CLIENT_ID")
        client_secret = os.environ.get("RESTAPI_DIRECTORY_CLIENT_SECRET")
        auth = get_wca_token(client_id, client_secret)
        url = f"{DIRECTORY_URL}/@vocabularies/collective.taxonomy.contact_category?b_size=1000000"
        categories_json = get_json(url, auth=auth)
        if categories_json is None or len(categories_json.get("items", [])) == 0:
            return SimpleVocabulary([])
        return SimpleVocabulary(
            [
                SimpleTerm(
                    value=elem["token"], token=elem["token"], title=elem["title"]
                )
                for elem in categories_json.get("items")
            ]
        )


DirectoryCategoriesVocabulary = DirectoryCategoriesVocabularyFactory()


class EventsTypesVocabularyFactory:
    @ram.cache(lambda *args: time() // (60 * 60))
    def __call__(self, context=None):
        client_id = os.environ.get("RESTAPI_EVENTS_CLIENT_ID")
        client_secret = os.environ.get("RESTAPI_EVENTS_CLIENT_SECRET")
        auth = get_wca_token(client_id, client_secret)
        url = f"{EVENTS_URL}/@vocabularies/imio.events.vocabulary.EventTypes"
        eventstypes_json = get_json(url, auth=auth)
        if eventstypes_json is None or len(eventstypes_json.get("items", [])) == 0:
            return SimpleVocabulary([])
        return SimpleVocabulary(
            [
                SimpleTerm(
                    value=elem["token"], token=elem["token"], title=elem["title"]
                )
                for elem in eventstypes_json.get("items")
            ]
        )


EventsTypesVocabulary = EventsTypesVocabularyFactory()


class EventsFromEntityVocabularyFactory:
    def __call__(self, context=None):
        remote_agendas_vocabulary = get_vocabulary(
            "imio.smartweb.vocabulary.RemoteAgendas"
        )
        selected_agendas = [
            f"selected_agendas={term.value}" for term in remote_agendas_vocabulary
        ]
        selected_agendas = "&".join(selected_agendas)
        today = date.today().isoformat()
        params = [
            selected_agendas,
            "portal_type=imio.events.Event",
            "metadata_fields=category",
            "metadata_fields=topics",
            "metadata_fields=start",
            "metadata_fields=end",
            "metadata_fields=has_leadimage",
            "metadata_fields=breadcrumb",
            "metadata_fields=UID",
            f"event_dates.query={today}",
            "event_dates.range=min",
            "b_size=1000000",
        ]
        # Keep default @search endpoint. Don't use @events endpoint.
        url = "{}/@search?{}".format(EVENTS_URL, "&".join(params))
        json_events = get_json(url)
        if json_events is None or len(json_events.get("items", [])) == 0:
            return SimpleVocabulary([])
        return SimpleVocabulary(
            [
                SimpleTerm(value=elem["UID"], title=elem["breadcrumb"])
                for elem in json_events.get("items")
            ]
        )


EventsFromEntityVocabulary = EventsFromEntityVocabularyFactory()


class NewsItemsFromEntityVocabularyFactory:
    def __call__(self, context=None):
        remote_newsfolders_vocabulary = get_vocabulary(
            "imio.smartweb.vocabulary.RemoteNewsFolders"
        )
        selected_newsfolders = [
            f"selected_news_folders={term.value}"
            for term in remote_newsfolders_vocabulary
        ]
        selected_newsfolders = "&".join(selected_newsfolders)
        params = [
            selected_newsfolders,
            "portal_type=imio.news.NewsItem",
            "metadata_fields=category",
            "metadata_fields=topics",
            "metadata_fields=has_leadimage",
            "metadata_fields=breadcrumb",
            "metadata_fields=UID",
            "b_size=1000000",
        ]
        url = "{}/@search?{}".format(NEWS_URL, "&".join(params))
        json_newsitems = get_json(url)
        if json_newsitems is None or len(json_newsitems.get("items", [])) == 0:
            return SimpleVocabulary([])
        return SimpleVocabulary(
            [
                SimpleTerm(value=elem["UID"], title=elem["breadcrumb"])
                for elem in json_newsitems.get("items")
            ]
        )


NewsItemsFromEntityVocabulary = NewsItemsFromEntityVocabularyFactory()


class AvailableInstanceBehaviorsVocabularyFactory:
    def __call__(self, context=None):
        sm = getSite().getSiteManager()
        utilities = sm.getUtilitiesFor(ITaxonomy)
        # getGeneratedName is the behavior name.
        return SimpleVocabulary(
            [
                SimpleTerm(value=utility.getGeneratedName(), title=utility.title)
                for utility_name, utility in utilities
                if utility.getShortName() not in ["procedure_category", "page_category"]
            ]
        )


AvailableInstanceBehaviorsVocabulary = AvailableInstanceBehaviorsVocabularyFactory()


class GalleryModeVocabularyFactory:
    def __call__(self, context=None):
        values = [
            ("gallery", _("Gallery")),
            ("carousel", _("Carousel")),
        ]
        terms = [
            SimpleVocabulary.createTerm(value[0], value[0], value[1])
            for value in values
        ]
        return SimpleVocabulary(terms)


GalleryModeVocabulary = GalleryModeVocabularyFactory()


class SendInBlueButtonPosVocabularyFactory:
    def __call__(self, context=None):
        bootstrap_css = [
            ("button_bottom", _("Bottom")),
            ("button_right", _("Right")),
        ]
        terms = [SimpleTerm(value=t[0], token=t[0], title=t[1]) for t in bootstrap_css]
        return SimpleVocabulary(terms)


SendInBlueButtonPosVocabulary = SendInBlueButtonPosVocabularyFactory()



# -*- coding: utf-8 -*-

from imio.smartweb.common.config import DESCRIPTION_MAX_LENGTH
from imio.smartweb.core.contents.sections.base import ISection
from imio.smartweb.core.contents.sections.base import Section
from imio.smartweb.locales import SmartwebMessageFactory as _
from zope.interface import implementer
from zope import schema


class ISectionExternalContent(ISection):
    """Marker interface and Dexterity Python Schema for SectionVideo"""

    description = schema.Text(
        title=_("Description"),
        description=_(
            "Use **text** to set text in bold. Limited to ${max} characters.",
            mapping={"max": DESCRIPTION_MAX_LENGTH},
        ),
        max_length=DESCRIPTION_MAX_LENGTH,
        required=False,
    )

    external_content_url = schema.URI(
        title=_("External content url"),
        required=True,
    )

    external_content_params = schema.Text(
        title=_("External content parameters"),
        required=False,
    )


@implementer(ISectionExternalContent)
class SectionExternalContent(Section):
    """SectionVideo class"""



# -*- coding: utf-8 -*-

from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from zope.component import queryMultiAdapter


class TestSectionExternalContent(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING

    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.page = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            id="page",
        )

    def test_unknow_service(self):
        sec = api.content.create(
            container=self.page, type="imio.smartweb.SectionExternalContent", id="sec"
        )
        sec.external_content_url = "https://kamoulox.be"
        section_view = queryMultiAdapter((sec, self.request), name="view")
        self.assertEqual(
            section_view.get_embed_external_content(),
            '<p class="unknow_service">Unknow service</p>',
        )

    def test_eaglebe_plugin(self):
        sec = api.content.create(
            container=self.page, type="imio.smartweb.SectionExternalContent", id="sec"
        )

        sec.external_content_url = ""
        section_view = queryMultiAdapter((sec, self.request), name="view")
        self.assertEqual(
            section_view.get_embed_external_content(),
            '<p class="unknow_service">Unknow service</p>',
        )

        sec.external_content_url = "https://app.eaglebe.com/auth/start"
        section_view = queryMultiAdapter((sec, self.request), name="view")
        self.assertEqual(
            section_view.get_embed_external_content(),
            '<iframe class="eaglebe" src="https://app.eaglebe.com/auth/start" scrolling="no" width="100%">',
        )

    def test_elloha_plugin(self):
        sec = api.content.create(
            container=self.page, type="imio.smartweb.SectionExternalContent", id="sec"
        )

        # without extra params
        sec.external_content_url = (
            "https://reservation.elloha.com/Scripts/widget-loader.min.js?v=42"
        )
        section_view = queryMultiAdapter((sec, self.request), name="view")
        self.assertEqual(
            section_view.get_embed_external_content(),
            '<div class="elloha elloha_error">With an elloha plugin, extra params must contain a dictionary with two keys : ConstellationWidgetContainer, Idoi</div>',
        )

        # with good extra params
        sec.external_content_params = """{
            "ConstellationWidgetContainer" : "11111111-1111-1111-1111-111111111111",
            "Idoi" : "22222222-2222-2222-2222-222222222222"
        }"""
        section_view = queryMultiAdapter((sec, self.request), name="view")
        result = '<div class="elloha"><div id="ConstellationWidgetContainer11111111-1111-1111-1111-111111111111" style="width:100%" data-id-projet="11111111111111111111111111111111">...</div></div><script type="text/javascript" src="https://reservation.elloha.com/Scripts/widget-loader.min.js?v=42/Scripts/widget-loader.min.js?v=42"></script><script type="text/javascript">var constellationWidgetUrl11111111111111111111111111111111, constellationTypeModule11111111111111111111111111111111;constellationWidgetUrl11111111111111111111111111111111 = "https://reservation.elloha.com/Widget/BookingEngine/11111111-1111-1111-1111-111111111111?idoi=22222222-2222-2222-2222-222222222222&culture=fr-FR";constellationTypeModule11111111111111111111111111111111=1; constellationWidgetLoad("ConstellationWidgetContainer11111111-1111-1111-1111-111111111111");constellationWidgetAddEvent(window, "resize", function () {constellationWidgetSetAppearance("ConstellationWidgetContainer11111111-1111-1111-1111-111111111111");});</script>'
        self.assertEqual(section_view.get_embed_external_content(), result)

        # good extra params / dict keys are case insensitive
        sec.external_content_params = """{
            "CONSTELLATIONWIDGETCONTAINER" : "11111111-1111-1111-1111-111111111111",
            "IDOI" : "22222222-2222-2222-2222-222222222222"
        }"""
        section_view = queryMultiAdapter((sec, self.request), name="view")
        result = '<div class="elloha"><div id="ConstellationWidgetContainer11111111-1111-1111-1111-111111111111" style="width:100%" data-id-projet="11111111111111111111111111111111">...</div></div><script type="text/javascript" src="https://reservation.elloha.com/Scripts/widget-loader.min.js?v=42/Scripts/widget-loader.min.js?v=42"></script><script type="text/javascript">var constellationWidgetUrl11111111111111111111111111111111, constellationTypeModule11111111111111111111111111111111;constellationWidgetUrl11111111111111111111111111111111 = "https://reservation.elloha.com/Widget/BookingEngine/11111111-1111-1111-1111-111111111111?idoi=22222222-2222-2222-2222-222222222222&culture=fr-FR";constellationTypeModule11111111111111111111111111111111=1; constellationWidgetLoad("ConstellationWidgetContainer11111111-1111-1111-1111-111111111111");constellationWidgetAddEvent(window, "resize", function () {constellationWidgetSetAppearance("ConstellationWidgetContainer11111111-1111-1111-1111-111111111111");});</script>'
        self.assertEqual(section_view.get_embed_external_content(), result)

        # with bad params
        sec.external_content_params = "kamoulox"
        section_view = queryMultiAdapter((sec, self.request), name="view")
        result = '<div class="elloha elloha_error">With an elloha plugin, extra params must contain a dictionary with two keys : ConstellationWidgetContainer, Idoi</div>'
        self.assertEqual(section_view.get_embed_external_content(), result)

        sec.external_content_params = """{
            "ConstellationWidgetContainer" : "11111111-1111-1111-1111-111111111111"
        }"""
        section_view = queryMultiAdapter((sec, self.request), name="view")
        result = '<div class="elloha elloha_error">With an elloha plugin, extra params must contain a dictionary with two keys : ConstellationWidgetContainer, Idoi</div>'
        self.assertEqual(section_view.get_embed_external_content(), result)

        sec.external_content_params = """{
            "CONSTELLATIONWIDGETCONTAINER" : "11111111-1111-1111-1111-111111111111",
            "IDOI" : "22222222-2222-2222-2222-222222222222"
        """
        section_view = queryMultiAdapter((sec, self.request), name="view")
        result = '<div class="elloha elloha_error">With an elloha plugin, extra params must contain a dictionary with two keys : ConstellationWidgetContainer, Idoi</div>'
        self.assertEqual(section_view.get_embed_external_content(), result)



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents import IDefaultPages
from zope.interface import noLongerProvides


def paste_default_page(obj, event):
    """Handle cut/copy & paste / rename of a default page (Procedure / Page)"""
    if event.oldParent == event.newParent and event.oldName != event.newName:
        # page was simply renamed
        return
    noLongerProvides(obj, IDefaultPages)
    obj.exclude_from_nav = False
    obj.reindexObject(idxs=["object_provides", "exclude_from_nav"])
    parent = event.oldParent
    if parent is not None:
        # page was cut from a folder
        parent.default_page_uid = None


def remove_default_page(obj, event):
    """Handle removal of a default page (Procedure / Page) from a folder"""
    noLongerProvides(obj, IDefaultPages)
    obj.exclude_from_nav = False
    obj.reindexObject(idxs=["object_provides", "exclude_from_nav"])
    folder = obj.aq_parent
    folder.default_page_uid = None



# -*- coding: utf-8 -*-

from eea.facetednavigation.browser.app.view import FacetedContainerView
from eea.facetednavigation.layout.interfaces import IFacetedLayout
from imio.smartweb.core.interfaces import IViewWithoutLeadImage
from imio.smartweb.core.utils import get_scale_url
from plone.app.contenttypes.browser.folder import FolderView
from zope.component import queryAdapter
from zope.interface import implementer


@implementer(IViewWithoutLeadImage)
class SmartwebFacetedContainerView(FacetedContainerView):
    """Faceted view without lead image"""


class FacetedView(FolderView):
    """Faceted common view"""

    @property
    def layout(self):
        context = self.context
        current_layout = queryAdapter(context, IFacetedLayout).layout
        return current_layout

    def show_images(self):
        return "with-images" in self.layout

    def is_video(self, item):
        return item.portal_type == "imio.smartweb.SectionVideo"

    def get_scale_url(self, item, scale="vignette"):
        if item.portal_type == "imio.smartweb.SectionGallery":
            images = item.getObject().listFolderContents()
            if not images:
                return ""
            scale_url = get_scale_url(images[0], self.request, "image", scale)
            return scale_url
        return get_scale_url(item, self.request, "image", scale)



# -*- coding: utf-8 -*-

from imio.smartweb.common.faceted.utils import configure_faceted
from imio.smartweb.core.behaviors.minisite import IImioSmartwebMinisite
from imio.smartweb.core.utils import safe_html
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.app.layout.navigation.interfaces import INavigationRoot
from zope.component import getMultiAdapter
from zope.lifecycleevent import ObjectRemovedEvent
from zope.globalrequest import getRequest

import os


def moved_folder(obj, event):
    """We use IObjectMovedEvent instead of IObjectAddedEvent because Minisite interface is not yet
    provided when we use IObjectAddedEvent"""
    if not IImioSmartwebMinisite.providedBy(obj):
        return
    if type(event) is ObjectRemovedEvent:
        # We don't have anything to do if minisite is being removed
        return
    parent = event.newParent
    if INavigationRoot.providedBy(parent):
        # Minisites must always be under navigation root
        if not IImioSmartwebMinisite.providedBy(parent):
            # We are not in another minisite, everything is good
            return
    request = getRequest()
    minisite_settings = getMultiAdapter((obj, request), name="minisite_settings")
    minisite_settings.disable()
    api.portal.show_message(
        _(
            "Your Folder was a minisite but this behaviour has been disabled with this action"
        ),
        request,
        type="warning",
    )


def added_collection(obj, event):
    faceted_config_path = "{}/faceted/config/collection.xml".format(
        os.path.dirname(__file__)
    )
    configure_faceted(obj, faceted_config_path)


def added_sectionhtml(obj, event):
    obj.html = safe_html(obj.html)


def modified_sectionhtml(obj, event):
    obj.html = safe_html(obj.html)
